#include veil:deferred_utils

uniform sampler2D DiffuseSampler0;
uniform sampler2D DiffuseDepthSampler;
uniform sampler2D Noise;
uniform float GameTime;
uniform vec3 heartPos;
uniform vec3 color;
uniform float timerOffset;
uniform float leftHanded;

in vec2 texCoord;
out vec4 fragColor;

// Helper function: Box intersection test
vec2 boxIntersection(in vec3 ro, in vec3 rd, vec3 boxSize, out vec3 outNormal) {
    vec3 m = 1.0 / rd;
    vec3 n = m * ro;
    vec3 k = abs(m) * boxSize;
    vec3 t1 = -n - k;
    vec3 t2 = -n + k;
    float tN = max(max(t1.x, t1.y), t1.z);
    float tF = min(min(t2.x, t2.y), t2.z);

    if (tN > tF || tF < 0.0) {
        return vec2(-1.0);  // No intersection
    }

    outNormal = (tN > 0.0) ? step(vec3(tN), t1) : step(t2, vec3(tF));
    outNormal *= -sign(rd);

    return vec2(tN, tF);
}

mat3 rotator(float angle){
    return mat3(
    cos(angle), 0.0, sin(angle),
    0.0, 1.0 ,0.0,
    -sin(angle),0.0,  cos(angle)

    );
}

vec3 viewPosFromDepth(float depth, vec2 uv) {
    float z = depth * 2.0 - 1.0;

    vec4 positionCS = vec4(uv * 2.0 - 1.0, z, 1.0);
    vec4 positionVS = VeilCamera.IProjMat * positionCS;
    positionVS /= positionVS.w;

    return positionVS.xyz;
}

vec3 viewDirFromUv(vec2 uv) {
    return (VeilCamera.IViewMat * vec4(normalize(viewPosFromDepth(1.0, uv)), 0.0)).xyz;
}


void main() {
    float speed = GameTime * 4000;
    vec4 baseColor = texture(DiffuseSampler0, texCoord);
    float depthSample = texture(DiffuseDepthSampler, texCoord).r;
    vec3 viewPos = viewPosFromDepthSample(depthSample, texCoord);
    vec3 rayDir = viewDirFromUv(texCoord) * rotator(speed * timerOffset) *  rotator(-5.41 - (leftHanded == 1 ? 1.75 : 0));


    vec3 camPos = VeilCamera.CameraPosition + VeilCamera.CameraBobOffset;


    vec3 handOffset = vec3(0,0,0);
    vec3 direction = vec3(VeilCamera.IViewMat[0][2] + handOffset.x, -VeilCamera.IViewMat[1][2] / 2 - sin(speed) / 5 - 0.6 , -VeilCamera.IViewMat[2][2] + handOffset.z) * rotator(speed * timerOffset);
    vec3 offsetDistance = vec3(3,1,3);
    vec3 posWorld = direction * offsetDistance + camPos;

    vec3 boxPosition = camPos - posWorld;


    vec3 iDontKnowWhatTheFuckIsANormal;

    float size = 0.3f;
    vec2 rayHitSmall = boxIntersection(boxPosition, rayDir, vec3(size, size, size), iDontKnowWhatTheFuckIsANormal);
    vec2 rayHitLarge = boxIntersection(boxPosition, rayDir, vec3(size * 1.2, size * 1.2, size * 1.2), iDontKnowWhatTheFuckIsANormal);

    bool hitSmall = rayHitSmall.x > 0.0;
    bool hitLarge = rayHitLarge.x > 0.0;

    if (hitSmall) {
        fragColor = vec4(vec3(1,1,1) , 1.0);
    } else if (hitLarge) {
        fragColor = vec4(vec3(color.x,color.y,color.z)*1.3  , 1.0);
    }
    else {
        fragColor = vec4(vec3(baseColor), 1.0);
    }
}