#include veil:space_helper


uniform sampler2D DiffuseSampler0;
uniform sampler2D DiffuseDepthSampler;
uniform sampler2D Noise;
uniform float GameTime;

in vec2 texCoord;
out vec4 fragColor;

float sphIntersect(vec3 ro, vec3 rd, vec4 sph){
    vec3 oc = ro - sph.xyz;
    float b = dot( oc, rd );
    float c = dot( oc, oc ) - sph.w*sph.w;
    float h = b*b - c;
    if( h<0.0 ){
        return - 1.0;
    }
    h = sqrt( h );
    return -b - h;
}
float depthSampleToWorldDepth(float depthSample) {
    float f = depthSample * 2.0 - 1.0;
    return 2.0 * VeilCamera.NearPlane * VeilCamera.FarPlane / (VeilCamera.FarPlane + VeilCamera.NearPlane - f * (VeilCamera.FarPlane - VeilCamera.NearPlane));
}

vec3 viewPosFromDepth(float depth, vec2 uv) {
    float z = depth * 2.0 - 1.0;

    vec4 positionCS = vec4(uv * 2.0 - 1.0, z, 1.0);
    vec4 positionVS = VeilCamera.IProjMat * positionCS;
    positionVS /= positionVS.w;

    return positionVS.xyz;
}

vec3 viewDirFromUv(vec2 uv) {
    return (VeilCamera.IViewMat * vec4(normalize(viewPosFromDepth(1.0, uv)), 0.0)).xyz;
}

void main() {
    vec3 baseColor = texture(DiffuseSampler0, texCoord).rgb;
    //float depth = texture(DiffuseDepthSampler, texCoord).r;
    //float worldDepth = depthSampleToWorldDepth(depth);

    vec3 rayDir = viewDirFromUv(texCoord);
    float distance = 0.0;
    float radiusOrbiter = 100;
    float speed = 1100;
    vec3 spherePos = vec3( cos(GameTime * speed)  * radiusOrbiter, 200.0,  sin(GameTime * speed) * radiusOrbiter);
    float sphereRadius = 50.0;

    float rayHit = sphIntersect(VeilCamera.CameraPosition, rayDir, vec4(spherePos, sphereRadius));

    if (rayHit > 0.0){
        fragColor = vec4(1,0,1, 1);
    } else {
        fragColor = vec4(vec3(baseColor), 1.0);
    }

}

