#include veil:space_helper
uniform sampler2D DiffuseSampler0;
uniform sampler2D DiffuseDepthSampler;
uniform sampler2D Noise;
uniform float GameTime;

in vec2 texCoord;
out vec4 fragColor;
float iTime = GameTime * 1000;

vec2 sphIntersect(vec3 ro, vec3 rd, vec3 ce, float ra )
{
    vec3 oc = ro - ce;
    float b = dot( oc, rd );
    float c = dot( oc, oc ) - ra*ra;
    float h = b*b - c;
    if( h<0.0 )
    return vec2(-1.0);
    h = sqrt( h );
    return vec2( -b-h, -b+h );
}

float random (in float x) {
    return fract(sin(x)*1e4);
}

float random (in vec2 st) {
    return fract(sin(dot(st.xy, vec2(12.9898,78.233)))* 43758.5453123);
}

float pattern(vec2 st, vec2 v, float t) {
    vec2 p = floor(st+v);
    return step(t, random(100.+p*.000001)+random(p.x)*0.5 );
}

void main() {
    vec3 baseColor = texture(DiffuseSampler0, texCoord).rgb;
    float depth = texture(DiffuseDepthSampler, texCoord).r;
    //float worldDepth = depthSampleToWorldDepth(depth);
    vec2 st = texCoord.xy;
    st.x /= VeilCamera.ProjMat[0][0];
    vec3 mouse = vec3(VeilCamera.IViewRotMat[0][0] ,VeilCamera.IViewRotMat[0][1], VeilCamera.IViewRotMat[0][2]);
    st.y *= mouse.y;
    vec2 grid = vec2(80.0,20.);
    st *= grid;

    vec2 ipos = floor(st);  // integer
    vec2 fpos = fract(st);  // fraction

    vec2 u_mouse = vec2(0.88, 0.11);
    vec2 u_resolution = vec2(1,1);

    vec2 vel = vec2(GameTime*1112.*max(grid.x,grid.y)); // time
    vel *= vec2(-0.1, 1.) * random(12.0+ipos.x); // direction

    // Assign a random value base on the integer coord
    vec2 offset = vec2(0, 0.);

    vec3 color = vec3(0.);
    color.r = pattern(st+offset,vel,0.5+u_mouse.x/u_resolution.x);
    color.g = pattern(st,vel,0.5+u_mouse.x/u_resolution.x);
    color.b = pattern(st-offset,vel,0.5+u_mouse.x/u_resolution.x);

    // Margins
    color *= step(0.,fpos.y);
    color = vec3(color.x * 11, color.y * 2, color.z * 1);
    vec3 finalColor = mix(baseColor * 2, color, 0.4);

    fragColor = vec4(finalColor, 1.0);
}